@@ -23,6 +23,7 @@ from .StorageTuple import StorageMemberTuple
 from .StorageTuple import StorageTuple
 from ._private.RelationGraph import RelationGraph
 from .tuples.git_commit_tuple import GitCommitTuple
+from .tuples.project_modified_tuple import ProjectModifiedTuple
 
 
 class TupleValidatorABC(ABC):
@@ -621,6 +622,27 @@ class ObjectStorageContext(ObjectStorageContextFileMixin):
         """
         raise NotImplementedError()
 
+    @abstractmethod
+    def numCommitsOnWorkingBranch(self) -> int:
+        """Number of Commits on the Working Branch
+
+        Get the number of commits on the __working__ branch
+
+        :return: int
+        """
+        raise NotImplementedError()
+
+    @abstractmethod
+    def changesSinceLastCommit(self) -> ProjectModifiedTuple:
+        """Changes Since Last Commit
+
+        Get a summary of the changes on the __working__ branch since
+        the previous squash and merge commit
+
+        :return: `ProjectModifiedTuple`
+        """
+        raise NotImplementedError()
+
     @abstractmethod
     def checkoutBranch(self, branchName: str) -> None:
         """Checkout Branch
@@ -697,6 +719,17 @@ class ObjectStorageContext(ObjectStorageContextFileMixin):
         """
         raise NotImplementedError()
 
+    @abstractmethod
+    def discardWorkingBranch(self) -> None:
+        """Discard Working Branch
+
+        Discard the working branch of the project to revert the changes made
+        since the previous commit and reload the project
+
+        :return: None
+        """
+        raise NotImplementedError()
+
     def _cascadeUpdateKey(
         self, itemForKeyUpdate: StorageTuple, newKey: str
     ) -> None:
