@@ -1,5 +1,6 @@
 import json
 import logging
+from collections import defaultdict
 from datetime import datetime
 from pathlib import Path
 from typing import List
@@ -10,13 +11,17 @@ from pathvalidate import sanitize_filepath
 from pygit2 import RemoteCallbacks
 
 from ..Exceptions import ChangesNotCommitedError
+from ..Exceptions import ItemNotFoundError
 from ..Exceptions import MergeConflict
 from ..Exceptions import NoChangesToCommitError
 from ..Exceptions import NoRemoteDefined
 from ..Exceptions import ProjectValidationError
+from ..StorageTuple import ItemStorageGroupEnum
 from .._private.readme_compiler import ReadmeCompiler
 from ..migration import getLatestRevision
 from ..tuples.git_commit_tuple import GitCommitTuple
+from ..tuples.project_modified_tuple import ModifiedItemDetails
+from ..tuples.project_modified_tuple import ProjectModifiedTuple
 
 
 logger = logging.getLogger(__name__)
@@ -456,16 +461,15 @@ class GitLibMixin:
 
         self._generateReadme()
 
-        working = self.__workingBranch
-        checkedOut = self.__checkedOutBranch
-        checkedLastCommit = next(self._repo.walk(checkedOut.target), None)
-
         # Walk the commits till we find the previous common point
-        commitsOnWorking = 0
-        for c in self._repo.walk(working.target):
-            if c.id == checkedLastCommit.id:
-                break
-            commitsOnWorking += 1
+        checkedLastCommit = next(
+            self._repo.walk(self.__checkedOutBranch.target), None
+        )
+        # NOTE: Do not inline this variable as self.__checkedOutBranch is a
+        # dynamic property and returns the currently checked out branch which
+        # could be __working__. We need a reference to the 'master' branch
+        # which should be the branch checked out at this point
+        checked = self.__checkedOutBranch
 
         self._repo.checkout(makeRef(ATTUNE_WORKING_BRANCH))
         # Create squash commit
@@ -497,9 +501,9 @@ class GitLibMixin:
         # Reload the reference to the working branch
         working = self.__workingBranch
 
-        self._repo.checkout(checkedOut.name)
-        checkedOut.set_target(working.target)
-        self._repo.reset(checkedOut.target, pygit2.GIT_RESET_HARD)
+        self._repo.checkout(checked.name)
+        checked.set_target(working.target)
+        self._repo.reset(checked.target, pygit2.GIT_RESET_HARD)
 
         # Delete the working branch. It will be recreated if there is a new
         # commit
@@ -511,6 +515,125 @@ class GitLibMixin:
         self._writeReadme(readmeContent)
         self.commit("Updated README.md")
 
+    def numCommitsOnWorkingBranch(self) -> int:
+        if ATTUNE_WORKING_BRANCH not in self._repo.branches:
+            return 0
+
+        working = self.__workingBranch
+        checkedOut = self.__checkedOutBranch
+        checkedLastCommit = next(self._repo.walk(checkedOut.target), None)
+
+        # Walk the commits till we find the previous common point
+        commitsOnWorking = 0
+        for c in self._repo.walk(working.target):
+            if c.id == checkedLastCommit.id:
+                break
+            commitsOnWorking += 1
+
+        return commitsOnWorking
+
+    def changesSinceLastCommit(self) -> ProjectModifiedTuple:
+        changes = ProjectModifiedTuple()
+        changes.commitsOnWorkingBranch = self.numCommitsOnWorkingBranch()
+
+        # Don't calculate the diff if __working__ does not exist
+        if changes.commitsOnWorkingBranch == 0:
+            return changes
+
+        # The keys of this dictionary correspond to folder names in the
+        # repository
+        changedItems = dict(
+            steps=defaultdict(lambda: None),
+            parameters=defaultdict(lambda: None),
+            files=defaultdict(lambda: None),
+        )
+
+        diff = self._repo.diff(
+            a=self.__checkedOutBranch.target, b=self.__workingBranch.target
+        )
+        for delta in diff.deltas:
+            change = delta.status_char()
+            fileName = delta.new_file.path
+
+            itemType, itemKey, *pathTail = fileName.split("/")
+            # We only include item modifications for now
+            # TODO: Expand this to include other changes
+            if itemType not in changedItems:
+                continue
+
+            modifiedFile = pathTail[-1]
+
+            if itemType == "steps":
+                itemGroup = ItemStorageGroupEnum.Step
+            elif itemType == "parameters":
+                itemGroup = ItemStorageGroupEnum.Parameter
+            elif itemType == "files":
+                itemGroup = ItemStorageGroupEnum.FileArchive
+            else:
+                raise NotImplementedError()
+
+            try:
+                item = self.getItem(itemGroup, itemKey)
+            except ItemNotFoundError:
+                changedItems[itemType][itemKey] = self._mergeItemChange(
+                    changedItems[itemType][itemKey],
+                    ModifiedItemDetails(
+                        key=itemKey,
+                        name=f"<key:{itemKey}>",
+                        changeStatus=change,
+                    ),
+                    modifiedFile,
+                )
+                continue
+            changedItems[itemType][itemKey] = self._mergeItemChange(
+                changedItems[itemType][itemKey],
+                ModifiedItemDetails(
+                    key=itemKey,
+                    name=item.name,
+                    changeStatus=change,
+                ),
+                modifiedFile,
+            )
+
+        changes.modifiedSteps = list(changedItems["steps"].values())
+        changes.modifiedFiles = list(changedItems["files"].values())
+        changes.modifiedParams = list(changedItems["parameters"].values())
+
+        return changes
+
+    def _mergeItemChange(
+        self,
+        prevChange: ModifiedItemDetails,
+        newChange: ModifiedItemDetails,
+        modifiedFile: str,
+    ) -> ModifiedItemDetails:
+        if prevChange is None:
+            # Removing files such as contents/ or comment.md from an item
+            # should be a modification change and not a deletion change
+            # Items are only deleted when their metadata.json and folder is
+            # deleted
+            if (
+                modifiedFile != "metadata.json"
+                and newChange.changeStatus == "D"
+            ):
+                newChange.changeStatus = "M"
+            return newChange
+
+        # We prefer the change to the metadata.json of an item
+        if modifiedFile == "metadata.json":
+            return newChange
+
+        if prevChange.changeStatus != "D" and newChange.changeStatus == "D":
+            newChange.changeStatus = prevChange.changeStatus
+        return newChange
+
+    def discardWorkingBranch(self):
+        if ATTUNE_WORKING_BRANCH not in self._repo.branches:
+            raise Exception("There are no changes to discard")
+
+        self._repo.branches.delete(ATTUNE_WORKING_BRANCH)
+        self.reloadProject()
+
     def getCommits(self) -> list[GitCommitTuple]:
         for commit in self._repo.walk(self.__checkedOutBranch.target):
             gitCommit = GitCommitTuple()
